// -*-C++-*-
/*!
 * @file  ConfigSVC_impl.cpp
 * @brief Service implementation code of Config.idl
 *
 */

#include <iostream>

#include "ConfigSVC_impl.h"
#include "RedundancySolutionCore.h"

/*
 * Example implementational code for IDL interface RedundancyConfig
 */
RedundancyConfigSVC_impl::RedundancyConfigSVC_impl()
{
  initVariables();
}

RedundancyConfigSVC_impl::~RedundancyConfigSVC_impl()
{
  m_comp = NULL;

  initVariables();
}


void RedundancyConfigSVC_impl::initVariables()
{
  //initialize variables
  m_mode = 0;
  m_type = RIGHT;
  m_isHoldOtherArm = false;
  
  
  LimitValue uplow = {1000,-1000};
  m_limit.rightElbow   = uplow;
  m_limit.leftElbow    = uplow;
  m_limit.lumbarRx     = uplow;
  m_limit.lumbarRy     = uplow;
  m_limit.lumbarRz     = uplow;
  m_limit.vehicleX     = uplow;
  m_limit.vehicleY     = uplow;
  m_limit.vehicleTheta = uplow;
}


/*
 * Methods corresponding to IDL attributes and operations
 */
ReturnID* RedundancyConfigSVC_impl::resetOriginalFrame()
{
  m_comp->resetOriginalFrame();
  cout << "resetOriginalFrame" << endl;



  ReturnID buf;
  ReturnID_var result;

  //default
  buf.id = 0;
  buf.comment = "restOriginalFrame";
  result = new ReturnID(buf);

  return result._retn();
}

ReturnID* RedundancyConfigSVC_impl::selectMode(CORBA::Short mode, RightLeft type, CORBA::Boolean isHoldOtherArm)
{
  m_mode = mode;
  m_type = type;
  m_isHoldOtherArm = isHoldOtherArm;

  m_comp->selectMode();

  cout << "selectMode" << endl;



  ReturnID buf;
  ReturnID_var result;
  
  //default
  buf.id = 1;
  buf.comment = "selectMode";
  result = new ReturnID(buf);
  
  return result._retn();
}

ReturnID* RedundancyConfigSVC_impl::selectRedundancy(const RedundantAxesMask& redundancy)
{
  m_comp->selectRedundancy(redundancy);
  cout << "selectRedundancy" << endl;



  ReturnID buf;
  ReturnID_var result;

  //default
  buf.id = 2;
  buf.comment = "selectRedundancy";
  result = new ReturnID(buf);
  
  return result._retn();
}

ReturnID* RedundancyConfigSVC_impl::setControlPointOffset(RightLeft type, const HgMatrix offset)
{
  m_comp->setControlPointOffset(type, offset);
  cout << "setControlPointOffset" << endl;



  ReturnID buf;
  ReturnID_var result;

  //default
  buf.id = 3;
  buf.comment = "setControlPointOffset";
  result = new ReturnID(buf);
  
  return result._retn();
}

ReturnID* RedundancyConfigSVC_impl::setRedundantAxesLimit(const RedundantLimit& limit)
{
  //m_comp->setRedundantAxesLimit(limit);
  
  m_limit = limit;

  cout << "setRedundantAxesLimit" << endl;



  ReturnID buf;
  ReturnID_var result;

  //default
  buf.id = 4;
  buf.comment = "setRedundantAxesLimit";
  result = new ReturnID(buf);
  
  return result._retn();
}



// End of example implementational code



