// -*-C++-*-
/*!
 * @file  ConfigSVC_impl.h
 * @brief Service implementation header of Config.idl
 *
 */



#ifndef CONFIGSVC_IMPL_H
#define CONFIGSVC_IMPL_H

#include "Config.hh"
 

/*!
 * @class RedundancyConfigSVC_impl
 * Example class implementing IDL interface RedundancyConfig
 */

class RedundancySolutionCore;

class RedundancyConfigSVC_impl
  : public virtual POA_RedundancyConfig,
    public virtual PortableServer::RefCountServantBase
{
private:
  // Make sure all instances are built on the heap by making the
  // destructor non-public
  //virtual ~RedundancyConfigSVC_impl();
  
public:
  /*!
   * @brief standard constructor
   */
  RedundancyConfigSVC_impl();
  /*!
   * @brief destructor
   */
  virtual ~RedundancyConfigSVC_impl();
  

  void initVariables();
  void init(RedundancySolutionCore *in_comp)
  {
    m_comp = in_comp;
  }
  

  // attributes and operations
  ReturnID* resetOriginalFrame();
  ReturnID* selectMode(CORBA::Short mode, RightLeft type, CORBA::Boolean isHoldOtherArm);
  ReturnID* selectRedundancy(const RedundantAxesMask& redundancy);
  ReturnID* setControlPointOffset(RightLeft type, const HgMatrix offset);
  ReturnID* setRedundantAxesLimit(const RedundantLimit& limit);
      

  // selectMode
  short m_mode;
  RightLeft m_type;
  bool m_isHoldOtherArm;
  

  // setRedundantAxesLimit
  RedundantLimit m_limit;


private:
  RedundancySolutionCore *m_comp;
};



#endif // CONFIGSVC_IMPL_H


