// -*- C++ -*-
/*!
 * @file  RedundancySolutionCore.h
 * @brief ModuleDescription
 * @date  $Date$
 *
 * $Id$
 */

#ifndef REDUNDANCYSOLUTIONCORE_H
#define REDUNDANCYSOLUTIONCORE_H

#include <rtm/Manager.h>
#include <rtm/DataFlowComponentBase.h>
#include <rtm/CorbaPort.h>
#include <rtm/DataInPort.h>
#include <rtm/DataOutPort.h>
#include <rtm/idl/BasicDataTypeSkel.h>
#include <rtm/idl/ExtendedDataTypesSkel.h>
#include <rtm/idl/InterfaceDataTypesSkel.h>


// user library
#include "callback_util.h"
#include "EngineRedundancy.h"
#include "JointName.h"


// Service implementation headers
#include "ConfigSVC_impl.h"


using namespace RTC;


CALLBACK_PREDEF(m_q_cur_in);
CALLBACK_PREDEF(m_xyzR_in);
//CALLBACK_PREDEF(m_redun_ref_in);


/*!
 * @class RedundancySolutionCore
 * @brief ModuleDescription
 *
 */
class RedundancySolutionCore
  : public RTC::DataFlowComponentBase
{
public:
  
  RedundancySolutionCore(RTC::Manager* manager);
  ~RedundancySolutionCore();
  
  
  // for RTC
  virtual RTC::ReturnCode_t onInitialize();
  virtual RTC::ReturnCode_t onActivated(RTC::UniqueId ec_id);
  virtual RTC::ReturnCode_t onDeactivated(RTC::UniqueId ec_id);
  virtual RTC::ReturnCode_t onExecute(RTC::UniqueId ec_id);
  
  
  // for service port
  void selectMode();
  void resetOriginalFrame() {
    engine.resetOriginalFrame();
  }
  void selectRedundancy(const RedundantAxesMask& redundancy) {
    engine.selectRedundancy(redundancy);
  }
  void setControlPointOffset(RightLeft type, const HgMatrix offset) {
    engine.setControlPointOffset(type, offset);
  }
  //void setRedundantAxesLimit(const RedundantLimit& limit);
  
  
protected:
  // Configuration variable declaration
  std::string m_conf_name;
  double m_double_dt;
  double m_double_limit;
  

  // DataInPort declaration
  TimedDoubleSeq m_q_cur_in;
  InPort<TimedDoubleSeq> m_q_cur_inIn;


  TimedDoubleSeq m_xyzR_in;
  InPort<TimedDoubleSeq> m_xyzR_inIn;


  TimedDoubleSeq m_redun_ref_in;
  InPort<TimedDoubleSeq> m_redun_ref_inIn;


  // DataOutPort declaration
  TimedDoubleSeq m_xyzR_out;
  OutPort<TimedDoubleSeq> m_xyzR_outOut;


  TimedDoubleSeq m_q_ref_out;
  OutPort<TimedDoubleSeq> m_q_ref_outOut;


  TimedDoubleSeq m_redun_cur_out;
  OutPort<TimedDoubleSeq> m_redun_cur_outOut;


  TimedDoubleSeq m_status_out;
  OutPort<TimedDoubleSeq> m_status_outOut;


  // CORBA Port declaration
  RTC::CorbaPort m_toRedundancyPort;

  
  // Service declaration
  RedundancyConfigSVC_impl m_toRedundancy;
 

 private:
  
  // for user library
  EngineRedundancy engine;
  JointName jointname;


  // to keep reference data
  bool firstCircle;
  TimedDoubleSeq m_xyzR_in_keeper;
  TimedDoubleSeq m_redun_ref_in_keeper;


  void setRefPosition(const TimedDoubleSeq& p);
  void setRefRedundant(const TimedDoubleSeq& phi);


  //_/_/_/_/_/_/for on write callback_/_/_/_/_/
  void _update_m_q_cur_in(const ConnectorInfo& info, const TimedDoubleSeq& q);
  CALLBACK_CLASS_MEMBER(m_q_cur_in);
  void _update_m_xyzR_in(const ConnectorInfo& info, const TimedDoubleSeq& d); 
  CALLBACK_CLASS_MEMBER(m_xyzR_in);


};

CALLBACK_CLASS(m_q_cur_in,TimedDoubleSeq,RedundancySolutionCore);
CALLBACK_CLASS(m_xyzR_in,TimedDoubleSeq,RedundancySolutionCore);


extern "C"
{
  DLL_EXPORT void RedundancySolutionCoreInit(RTC::Manager* manager);
};

#endif // REDUNDANCYSOLUTIONCORE_H
