/**********************************************************************
  標準モデル用ライブラリ
  基本的に仮想ジョイントの管理はパーツ毎に行う
**********************************************************************/

#ifndef STANDARD_MODEL_H_INCLUDE
#define STANDARD_MODEL_H_INCLUDE


#include <string>
#include <ostream>
#include <vector>
#include <boost/intrusive_ptr.hpp>
#include <boost/shared_ptr.hpp>


// OpenHRP3
#include <hrpModelHeaders.h>


namespace tohoku {

  class VirtualJoint;
  class StandardModelPart;

  class StandardModel {

  public:
    StandardModel();
    ~StandardModel();


    void addPart(StandardModelPart* part);
    void updateStandardModel();


    inline void calcForwardKinematics() {
      body->calcForwardKinematics();
    }


    inline void setRealModel(OpenHRP::BodyPtr realModel) {
      body = realModel;
    }


    inline VirtualJoint* virtualJoint(int index) const {
      return virtualJoints[index];
    }
    VirtualJoint* virtualJoint(const std::string& name) const ;


    inline StandardModelPart* part(int index) const {
      return parts[index];
    }
    StandardModelPart* part(const std::string& name) const;


    inline int numVirtualJoints() const {
      return virtualJoints.size();
    }
    

    inline int numParts() const {
      return parts.size();
    }


    OpenHRP::BodyPtr body;

  protected:
    std::vector<VirtualJoint*> virtualJoints;
    std::vector<StandardModelPart*> parts;


  private:
    typedef std::map<std::string, VirtualJoint*> NameToVirtualJointMap;
    NameToVirtualJointMap nameToVirtualJointMap;
    
    typedef std::map<std::string, StandardModelPart*> NameToPartMap;
    NameToPartMap nameToPartMap;
  };

};


#endif
