#include "StandardModelPart.h"
#include "VirtualJoint.h"

#include <algorithm>

using namespace std;
using namespace tohoku;
using namespace OpenHRP;


StandardModelPart::StandardModelPart()
{
  name = "";

  virtualJoints.clear();

  num_real_links  = 0;
  num_real_joints = 0;

  nameToVirtualJointMap.clear();
}


StandardModelPart::StandardModelPart(string in_name)
{
  name = in_name;

  virtualJoints.clear();

  num_real_links  = 0;
  num_real_joints = 0;

  nameToVirtualJointMap.clear();
}


StandardModelPart::~StandardModelPart()
{
  name = "";

  virtualJoints.clear();

  num_real_links  = 0;
  num_real_joints = 0;

  nameToVirtualJointMap.clear();
}


void StandardModelPart::addVirtualJoint(VirtualJoint* virtualJoint)
{
  virtualJoints.push_back(virtualJoint);

  num_real_links  += virtualJoint->numLinks();
  num_real_joints += virtualJoint->numJoints();

  nameToVirtualJointMap[virtualJoint->name] = virtualJoint;
}


void StandardModelPart::calcJacobian(dmatrix& out_J, VirtualJoint* target) const
{
  const int n = virtualJoints.size();
  out_J.resize(6, n, false);
  out_J = dzeromatrix(6,n);
  
  if(n > 0){
    
    Link* targetLink;
    if(target) {
      targetLink = target->endLink();
    }
    else {
      targetLink = virtualJoints.back()->endLink();
    }
    
    
    for(int i = 0; i < n; ++i){
      
      VirtualJoint* vlink = virtualJoints[i];
      
      for(int j = 0; j < vlink->numJoints(); j++) {
	Link*  link  = vlink->joint(j);
	double ratio = vlink->ratio(j);
	
	switch(link->jointType){
	  
	case Link::ROTATIONAL_JOINT:
	  {
	    vector3 omega(link->R * link->a);
	    vector3 arm(targetLink->p - link->p);
	    vector3 dp(cross(omega, arm));
	    
	    for(int k = 0; k < 3; k++) {
	      out_J(k, i)   += (dp(k) * ratio);
	      out_J(k+3, i) += (omega(k) * ratio);
	    }
	  }
	  break;
	  
	case Link::SLIDE_JOINT:
	  {
	    vector3 dp(link->R * link->d);
	    for(int k = 0; k < 3; k++) {
	      out_J(k, i) += (dp(k) * ratio);
	    }
	    out_J(3, i) += 0.0;
	    out_J(4, i) += 0.0;
	    out_J(5, i) += 0.0;
	  }
	  break;
	  
	default:
	  for(int k = 0; k < 6; ++k){
	    out_J(k, i) += 0.0;
	  }
	}
      }
    }
  }
}


VirtualJoint* StandardModelPart::virtualJoint(const std::string& name) const
{
  NameToVirtualJointMap::const_iterator p = nameToVirtualJointMap.find(name);
  return (p != nameToVirtualJointMap.end()) ? p->second : 0;
}
