/**********************************************************************
  仮想ジョイント用ライブラリ
**********************************************************************/

#ifndef VIRTUAL_JOINT_H_INCLUDED
#define VIRTUAL_JOINT_H_INCLUDED

#include <string>
#include <ostream>
#include <vector>
#include <boost/intrusive_ptr.hpp>
#include <boost/shared_ptr.hpp>


// OpenHRP3
#include <hrpModelHeaders.h>


namespace tohoku {

  class VirtualJoint {
  
  public:
    VirtualJoint();
    VirtualJoint(std::string in_name);
    ~VirtualJoint();
    
    void addMember(OpenHRP::Link* link, double ratio=1.0);

    std::string name;


    inline double q() const {
      return virtualAngle;
    }
    double q(const double in_q);


    inline bool empty() const {
      //return realLinks.empty();
      return realJoints.empty();
    }
    

    inline int numLinks() const {
      return realLinks.size();
    }


    inline int numJoints() const {
      return realJoints.size();
    }


    inline double ratio(const int index) const {
      return ratios[index];
    }
    
    
    inline OpenHRP::Link* link(const int index) const {
      return realLinks[index];
    }


    inline OpenHRP::Link* joint(const int index) const {
      return realJoints[index];
    }
    

    inline OpenHRP::Link* rootLink() const {
      return realLinks.front();
    }
    

    inline OpenHRP::Link* endLink() const {
      return realLinks.back();
    }


  protected:
    double virtualAngle;

    std::vector<OpenHRP::Link*> realLinks;

    std::vector<OpenHRP::Link*> realJoints;
    std::vector<double> ratios;

  private:
    VirtualJoint& operator=(const VirtualJoint& virtualJoint);
    
  };
  
};


#endif
